local HC = require 'HardonCollider'
local loader = require "AdvTiledLoader/Loader"
require 'functions.lua'
require 'player.lua'

loader.path = "maps/"

local allSolidTiles

function love.load()

    global = {}
    global.tx = 0                   -- X translation of the screen
    global.ty = 0                   -- Y translation of the screen

    -- initialize library
    Collider = HC(32, on_collision, collision_stop)
    bang = string.format("lol")
    time = 0
    time2 = 0
    animtime = 0
    animtime2 = 0
    animknife = 1
    ztime = 0

    fps = 0
    fpsCount = 0
    fpsTime = 0

    --particle
    flashimg = love.graphics.newImage("particle/flash.png")
    flash = love.graphics.newParticleSystem( flashimg, 50 )
    emitterpos = Collider:addPoint(10+28,25)
    flash:setEmissionRate          (50)
    flash:setLifetime              (0.1)
    flash:setParticleLife          (0.2)
    flash:setPosition              (50, 50)
    flash:setDirection             (0)
    flash:setSpread                (1)
    flash:setSpeed                 (50, 100)
    flash:setGravity               (0)
    flash:setRadialAcceleration    (10)
    flash:setTangentialAcceleration(0)
    flash:setSize                  (0.5)
    flash:setSizeVariation         (0.5)
    flash:setRotation              (0)
    flash:setSpin                  (0)
    flash:setSpinVariation         (0)
    flash:stop()

    --add the dude
    dude = love.graphics.newImage("char/dude.png")
    smp = love.graphics.newImage("char/smp.png")
    selImg = love.graphics.newImage("selection/sel.png")
    knife1 = love.graphics.newImage("char/knife_1.png")
    knife2 = love.graphics.newImage("char/knife_2.png")
    knife3 = love.graphics.newImage("char/knife_3.png")
    bulletimg = love.graphics.newImage("particle/bullet.png")
    currwep = "smp"
    speedx = 0
    speedy = 0
    health = 100
    kills = 0
    circ = Collider:addRectangle(10, 10, 15, 24)

    --add zombie
    zombieimg = love.graphics.newImage("enemies/zombie.png")
    zombies = {}

    gw, gh = love.graphics.getWidth(), love.graphics.getHeight()
    bullets = {}

    map = loader.load("level.tmx")
    allSolidTiles = findSolidTiles(map)
    Collider:setPassive(circ)
    Collider:setGhost(circ)
end

function love.update(dt)
    xc, yc = circ:center()
    global.tx = xc - 800/1.5/2
    global.ty = yc - 600/1.5/2
    map:setDrawRange(xc - 800/1.5/2,yc - 600/1.5/2,love.graphics.getWidth()/1.5, love.graphics.getHeight()/1.5)

    fpsCount = fpsCount+1
    fpsTime = fpsTime + dt
    if fpsTime >= 1 then
        fps = fpsCount
        fpsTime = 0
        fpsCount = 0
    end

    bullet_update(dt)
    zombie_update(dt)
    Collider:update(dt)
    player_update(dt)
    xc, yc = circ:center()
end

function love.draw()
    local ftx, fty = math.floor(global.tx), math.floor(global.ty)
    love.graphics.push()
    love.graphics.scale(1.5)

    love.graphics.translate(-ftx, -fty)

    love.graphics.setColor(255,255,255)
    map:draw()
    --circ:draw("fill")
    -- print messages
    love.graphics.print("Health:",0,0)
    love.graphics.print(health,45,0)
    love.graphics.print("FPS:",0,15)
    love.graphics.print(fps,27,15)
    love.graphics.print("Kills:",0,30)
    love.graphics.print(kills,27,30)

    love.graphics.setColor(255,255,255)
    love.graphics.draw(flash, 0, 0)
    --emitterpos:draw('line')
    love.graphics.draw(dude, xc, yc, circAngle-math.pi, 1, 1, 12, 16)
    if currwep == "smp" then
        love.graphics.draw(smp, xc, yc, circAngle-math.pi, 1, 1, 12, 16)
    elseif currwep == "knife" then 
        if animknife == 1 then 
            love.graphics.draw(knife1, xc, yc, circAngle-math.pi, 1, 1, 12, 16)
        elseif animknife == 2 then
            love.graphics.draw(knife2, xc, yc, circAngle-math.pi, 1, 1, 12, 16)
        elseif animknife == 3 then 
            love.graphics.draw(knife3, xc, yc, circAngle-math.pi, 1, 1, 12, 16)
        end
    end 

    for i, b in ipairs(bullets) do
        love.graphics.setColor(0,0,0)
        local bx,by = b:center()
        love.graphics.draw(bulletimg, bx,by, _, 0.7,0.7)
    end

    for i, z in ipairs(zombies) do
        love.graphics.setColor(255,255,255)
        --z:draw("fill")
        zx, zy = z:center()
        zombAngle = getAngle(zx,zy, xc,yc)
        love.graphics.draw(zombieimg, zx, zy, zombAngle, 1, 1, 9, 16)
    end
    love.graphics.pop()

end

-- this is called when two shapes stop colliding
function collision_stop(dt, shape_a, shape_b)

end

function on_collision(dt, shape_a, shape_b, mtv_x, mtv_y)

end

function zombie_update(dt)
    for i, z in ipairs(zombies) do
        zx, zy = z:center()
        zombAngle = getAngle(zx,zy, xc,yc)
        --z:setRotation(zombAngle-math.pi)
        local dir = zombAngle
        z.xvel = math.cos(dir) * z.speed
        z.yvel = math.sin(dir) * z.speed
        z:move(z.xvel * dt, z.yvel * dt)

        if z:collidesWith(circ) then 
            local collbol, collmtv = z:collidesWith(circ)
            z:move(collmtv.x, collmtv.y)
            if love.keyboard.isDown("lshift") then 
                z.speed = -200
                z.push = true
            else 
                health = health - 0.5
            end
        end
        
        if z.push == true then 
            z.time = z.time + dt 
            if z.time > 0.2 then 
                z.speed = 40
                z.push = false
                z.time = 0
            end
        end

        if z.health < 1 then 
            Collider:remove(z)
            table.remove(zombies, i)
            kills = kills + 1
            break
        end

        if #zombies > 1 then
            for k = #zombies, i+1, -1 do 
                local k = zombies[k]
                if z:collidesWith(k) then
                    local collbol, collmtv = z:collidesWith(k)
                    z:move(collmtv.x, collmtv.y)
                end
            end
        end

        for i = #bullets, 1, -1 do
            local b = bullets[i]
            if b:collidesWith(z) then 
                z.health = z.health - 1
                table.remove(bullets, i)
                Collider:remove(b)
            end 
        end

        for j = #allSolidTiles, 1, -1 do
            local t = allSolidTiles[j]
            if z:collidesWith(t) then
                local collbol, collmtv = z:collidesWith(t)
                z:move(collmtv.x, collmtv.y)
            end
        end
    end
end

function bullet_update(dt)
    Collider:update(dt)
    for i = #bullets, 1, -1 do -- reverse-order trick
        local b = bullets[i]
        local bx, by = b:center()
        if bx < 0 or bx > gw or by < 0 or by > gh then
            table.remove(bullets, i)
            Collider:remove(b)
        else
            b:move(b.xvel * dt, b.yvel * dt)
            bx2, by2 = b:center()
        end
        for j = #allSolidTiles, 1, -1 do
            local t = allSolidTiles[j]
            if b:collidesWith(t) then 
                table.remove(bullets, i)
                Collider:remove(b)
            end 
        end
    end
end 

function player_update(dt)
    --bullet stuff
    circ:move(speedx*dt,speedy*dt)
    emitterpos:move(speedx*dt,speedy*dt)

    for j = #allSolidTiles, 1, -1 do
        local t = allSolidTiles[j]
        if circ:collidesWith(t) then
            local collbol, collmtv = circ:collidesWith(t)
            circ:move(collmtv.x, collmtv.y)
            emitterpos:move(collmtv.x, collmtv.y)
        end
    end

    lol = dt
    x, y = love.mouse.getPosition( )
    x = x/1.5
    y = y/1.5
    xc, yc = circ:center()

    circAngle = getAngle(x,y ,xc,yc)
    circ:setRotation(circAngle-math.pi)

    -- particle flash
    emitterpos:setRotation(circAngle-math.pi, xc, yc)
    xe, ye = emitterpos:center()
    flash:setPosition(xe, ye)
    flash:setDirection(emitterpos:rotation())
    

    Collider:update(dt)
    player_move(dt)
    player_shoot(dt)
end